


/*		MODULE FOR CRYSTAL IMAGE 0.85 BETA
		This program is copyrighted material.
		(C) 2002-2010 by Mark A. Haidekker, all rights reserved.
		
		This program is free software: you can redistribute it and/or modify
		it under the terms of the GNU General Public License as published by
		the Free Software Foundation, either version 3 of the License, or
		(at your option) any later version.

		This program is distributed in the hope that it will be useful,
		but WITHOUT ANY WARRANTY; without even the implied warranty of
		MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE:

		THERE IS NO WARRANTY FOR THE PROGRAM, TO THE EXTENT PERMITTED BY
		APPLICABLE LAW.  EXCEPT WHEN OTHERWISE STATED IN WRITING THE COPYRIGHT
		HOLDERS AND/OR OTHER PARTIES PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY
		OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING, BUT NOT LIMITED TO,
		THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
		PURPOSE.  THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM
		IS WITH YOU.  SHOULD THE PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF
		ALL NECESSARY SERVICING, REPAIR OR CORRECTION.

		IN NO EVENT UNLESS REQUIRED BY APPLICABLE LAW OR AGREED TO IN WRITING
		WILL ANY COPYRIGHT HOLDER, OR ANY OTHER PARTY WHO MODIFIES AND/OR CONVEYS
		THE PROGRAM AS PERMITTED ABOVE, BE LIABLE TO YOU FOR DAMAGES, INCLUDING ANY
		GENERAL, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES ARISING OUT OF THE
		USE OR INABILITY TO USE THE PROGRAM (INCLUDING BUT NOT LIMITED TO LOSS OF
		DATA OR DATA BEING RENDERED INACCURATE OR LOSSES SUSTAINED BY YOU OR THIRD
		PARTIES OR A FAILURE OF THE PROGRAM TO OPERATE WITH ANY OTHER PROGRAMS),
		EVEN IF SUCH HOLDER OR OTHER PARTY HAS BEEN ADVISED OF THE POSSIBILITY OF
		SUCH DAMAGES.

		For the full text of the license, please see the file COPYING
		in the project's main directory, or read it on the Web at
		http://www.gnu.org/copyleft/gpl.html

 */



#include <gtk/gtk.h>
#include <gdk/gdk.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>



#include "../src/defs.h"
#include "../src/dlgbox.h"
#include "../src/functions.h"
#include "../src/findclust.h"
#include "../src/interface.h"
#include "../src/dso.h"
#include "../src/macro.h"
#include "../src/gtkhelpers.h"
#include "../src/special1.h"
#include "../src/imgbuf.h"
#include "../src/application.h"
#include "../src/secondwnd.h"
#include "../src/tiffstuff.h"



/* 

	An empty modlue stub. It contains placeholders for
	all components of a Crystal Image module, which are:

	1)	The actual module functionality with a dummy
		function mod_dummy

	2)	The menu hookup function that enters the module
		into the "special" menu

	3)	The Macro interpreter hookup that adds a macro
		function "dummy" to the macro interpreter
*/


/*	The module can be compiled with the following entry in makefile. Just type make 0module.dso

# Handled out of the ordinary. This line compiles the dummy module (dso sample function.
# Once it is compiled, manually copy the dso into your dso directory, which is
# either /usr/local/cimage/dso or ~/dso

0module.dso: 0module.c
	$(CC) $(GTK_CFLAGS) $(DEBUGFLG) -c -fPIC 0module.c
	$(CCLD) $(imager_LDADD) -shared -fPIC -o 0module.dso 0module.o
	chmod a-x 0module.dso

*/




/*          ###     ###    ####    #####           #####   #   #   #   #    ###    #####    ###     ###    #   #          */
/*         #   #   #   #   #   #   #               #       #   #   #   #   #   #     #       #     #   #   #   #          */
/*         #       #   #   #   #   #               #       #   #   ##  #   #         #       #     #   #   ##  #          */
/*         #       #   #   ####    ####            ####    #   #   # # #   #         #       #     #   #   # # #          */
/*         #       #   #   # #     #               #       #   #   #  ##   #         #       #     #   #   #  ##          */
/*         #   #   #   #   #  #    #               #       #   #   #   #   #   #     #       #     #   #   #   #          */
/*          ###     ###    #   #   #####           #        ###    #   #    ###      #      ###     ###    #   #          */
/*                                                                                                                        */


/*
	The operator:
	This function performs the actual module function, whatever it is.
	It gets passed an image to work on (img).
	Upon exit, img is modified according by the operator.
*/


void mod_dummy (image_type *img)
{
image_type tempimg;			/* If temporary storage is needed */

	dp (1, "Executing the dummy dso operator\n");		/* Watch your console output, have debuglevel >= 1 */


	/* handback (img, &tempimg);		 If temporary storage is needed */

	dp (1, "Exiting the dummy dso operator\n");

}



/**************************************************************************************************************************/

/*         ####     ###     ###             ###    #####   ####    #   #    ###    #####   #   #   ####    #####          */
/*         #   #   #   #   #   #           #   #     #     #   #   #   #   #   #     #     #   #   #   #   #              */
/*         #   #   #       #   #           #         #     #   #   #   #   #         #     #   #   #   #   #              */
/*         #   #    ###    #   #            ###      #     ####    #   #   #         #     #   #   ####    ####           */
/*         #   #       #   #   #               #     #     # #     #   #   #         #     #   #   # #     #              */
/*         #   #   #   #   #   #           #   #     #     #  #    #   #   #   #     #     #   #   #  #    #              */
/*         ####     ###     ###             ###      #     #   #    ###     ###      #      ###    #   #   #####          */
/*                                                                                                                        */




/**************************************************************

	This is the macro interpreter function that hooks up
	to the macro interpreter. A pointer to this function gets
	passed to the macro registry function.


**************************************************************/



variable_type mac_dummy_function (unsigned char* s, int* i)
{
	variable_type v,r;
	int j;
	float level;

	j = *i; 
	if (!isalpha(s[j])) macro_error (1,j);	
	v = expr (s,&j, MT_IMG);					/* Evaluate the first expression and get image into v.imgptr */

	/* IF you have additional parameters, use something like this...
	if (s[j++]!=',') macro_error (1,j);
	r = expr (s,&j, MT_REAL); level = r.numvalue;
	...*/

	while (s[j]!=')') j++;					/* Skip to closing parentheses, end expression */
	*i = j+1;

	/* Prepare to use the operator */

	r = nullvar;
	mod_dummy (&v.imgptr);

	return v;		/* Return the modified image in the structure of variable v */
}


/**************************************************************

	This is the infrastructure we use to register with the 
	main program. The pointer "menufunction" points to the 
	main entry (the function to be called when the menu is 
	activated). Initmodule is the one function that the 
	main program calls (the name is fixed!) to initialize
	the module and to register the module with the menu.

***************************************************************/

void (*menufunction1)();		/* A pointer to the menu callback function for the main menu */


	/* A pointer to the macro function for the macro interpreter */
variable_type (*macrofunction)(unsigned char* s, int* i);


/* This function corresponds to the driver functions in callbacks.c. Its only
	purpose is to call the actual operator.
	For multiple operators, each operator has one menu function pointer and one of
	these driver functions */

void module_dummy_driver ()
{

	if (file_ok) 
	{
		mod_dummy (&mainimg);		/* Execute the module's operator with mainimage */
	}
}




void initmodule ()
{

	dp (1, "Dummy module: Initializer called\n");
	
	/* Register this module with the main menu */
	
	menufunction1 = &module_dummy_driver;
	dso_menu_register ("Dummy module", menufunction1);


	/* Register the module with the macro interpreter. Note that DUMMYFN is the
		token that the macro interpreter uses. The function takes an image
		as argument and returns the modified image in the sense of
		modimg = DUMMYFN (origimg); */
	
	macrofunction = &mac_dummy_function;
	register_dso_macro ("DUMMYFN",1,MT_IMG, macrofunction);
	

	
}

